// ICU
#include <unicode/unistr.h>

extern const UnicodeString UKRAINIAN_LATIN_BGN_TRANSLITERATION_RULES (
"::[ЄІЇА-ЩЬЮ-щьюяєіїҐґ’];"
"::NFD(NFC);"
"А > A;"
"а > a;"
"Б > B;"
"б > b;"
"В > V;"
"в > v;"
"Г > H;"
"г > h;"
"Ґ > G;"
"ґ > g;"
"Д > D;"
"д > d;"
"Е > E;"
"е > e;"
"{Є}[[б-джзй-нп-тф-щьґ’][аеиоуюяєії]] > Ye;"
"Є > YE;"
"є > ye;"
"{Ж}[[б-джзй-нп-тф-щьґ’][аеиоуюяєії]] > Zh;"
"Ж > ZH;"
"ж > zh;"
"ЗГ > Z·H;"
"Зг > Z·h;"
"зг > z·h;"
"З > Z;"
"з > z;"
"И > Y;"
"и > y;"
"І > I;"
"і > i;"
"{Ї}[[б-джзй-нп-тф-щьґ’][аеиоуюяєії]] > Yi;"
"Ї > YI;"
"ї > yi;"
"Й > I;"
"й > i;"
"КГ > K·H;"
"Кг > K·h;"
"кг > k·h;"
"К > K;"
"к > k;"
"Л > L;"
"л > l;"
"М > M;"
"м > m;"
"Н > N;"
"н > n;"
"О > O;"
"о > o;"
"П > P;"
"п > p;"
"Р > R;"
"р > r;"
"СГ > S·H;"
"Сг > S·h;"
"сг > s·h;"
"С > S;"
"с > s;"
"ТС > T·S;"
"Тс > T·s;"
"тс > t·s;"
"Т > T;"
"т > t;"
"У > U;"
"у > u;"
"Ф > F;"
"ф > f;"
"{Х}[[б-джзй-нп-тф-щьґ’][аеиоуюяєії]] > Kh;"
"Х > KH;"
"х > kh;"
"ЦГ > TS·H;"
"Цг > Ts·h;"
"цг > ts·h;"
"{Ц}[[б-джзй-нп-тф-щьґ’][аеиоуюяєії]] > Ts;"
"Ц > TS;"
"ц > ts;"
"{Ч}[[б-джзй-нп-тф-щьґ’][аеиоуюяєії]] > Ch;"
"Ч > CH;"
"ч > ch;"
"ШЧ > SH·CH;"
"Шч > Sh·ch;"
"шч > sh·ch;"
"{Ш}[[б-джзй-нп-тф-щьґ’][аеиоуюяєії]] > Sh;"
"Ш > SH;"
"ш > sh;"
"{Щ}[[б-джзй-нп-тф-щьґ’][аеиоуюяєії]] > Shch;"
"Щ > SHCH;"
"щ > shch;"
"{Ю}[[б-джзй-нп-тф-щьґ’][аеиоуюяєії]] > Yu;"
"Ю > YU;"
"ю > yu;"
"{Я}[[б-джзй-нп-тф-щьґ’][аеиоуюяєії]] > Ya;"
"Я > YA;"
"я > ya;"
"Ь > ʹ;"
"ь > ʹ;"
"’ > ʺ;"
);
