#ifndef __OPENTREP_BAS_FLOAT_UTILS_HPP
#define __OPENTREP_BAS_FLOAT_UTILS_HPP

// //////////////////////////////////////////////////////////////////////
// Import section
// //////////////////////////////////////////////////////////////////////
// OpenTrep
#include <opentrep/basic/float_utils_google.hpp>

/**
 * Utility class for floating point operations.
 *
 * \see
 * <ul>
 *  <li>http://stackoverflow.com/questions/17333/most-effective-way-for-float-and-double-comparison</li>
 *  <li>http://www.lomont.org/Math/Papers/2005/CompareFloat.pdf</li>
 *  <li>http://www.randydillon.org/Papers/2007/everfast.htm</li>
 * </ul>
 */

#endif // __OPENTREP_BAS_FLOAT_UTILS_HPP
