import L from "leaflet";
export declare function instanceOfLatLngLiteral(object: any): object is L.LatLngLiteral;
export declare function instanceOfLatLngTuple(object: any): object is L.LatLngTuple;
export declare function instanceOfLatLngExpression(object: any): object is L.LatLngExpression;
export declare function latlngExpressiontoLatLng(input: L.LatLngExpression): L.LatLng;
export declare function latlngExpressionArraytoLatLngArray(input: L.LatLngExpression[] | L.LatLngExpression[][]): L.LatLng[][];
