<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
  {% block head %}

    {% block meta %}
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
    <meta name="robots" content="NONE,NOARCHIVE"/>
    {% endblock %}

    {% block jscript %}
    {% endblock %}

    {% block style %}
	{% endblock %}

    <title>{% block title %}Title{% endblock %}</title>

  {% endblock %}
  </head>

  <body class="{% block bodyclass %}{% endblock %}"
		{% block bodyoptions %}{% endblock %}>
	{% block content %}
	{% endblock %}

	{% block googleanalytics %}
	<script>
      (function(i,s,o,g,r,a,m){
      i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
      (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
      m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
      })(window,document,'script','//www.google-analytics.com/analytics.js','ga');

      ga('create', 'UA-46880930-1', 'transport-search.org');
      ga('send', 'pageview');

      var dimensionValue = '3';
      ga('set', 'dimension1', dimensionValue);

      var metricValue = '5';
      ga('set', 'metric1', metricValue);
	</script>
	{% endblock %}

  </body>
</html>
